@@ -1,16 +1,11 @@
-from __future__ import annotations
+import numpy as np
 
 from ._array_object import Array
 from ._dtypes import _integer_dtypes
+from ._flags import requires_api_version
 
-from typing import TYPE_CHECKING
 
-if TYPE_CHECKING:
-    from typing import Optional
-
-import numpy as np
-
-def take(x: Array, indices: Array, /, *, axis: Optional[int] = None) -> Array:
+def take(x: Array, indices: Array, /, *, axis: int | None = None) -> Array:
     """
     Array API compatible wrapper for :py:func:`np.take <numpy.take>`.
 
@@ -25,3 +20,15 @@ def take(x: Array, indices: Array, /, *, axis: Optional[int] = None) -> Array:
     if x.device != indices.device:
         raise ValueError(f"Arrays from two different devices ({x.device} and {indices.device}) can not be combined.")
     return Array._new(np.take(x._array, indices._array, axis=axis), device=x.device)
+
+
+@requires_api_version('2024.12')
+def take_along_axis(x: Array, indices: Array, /, *, axis: int = -1) -> Array:
+    """
+    Array API compatible wrapper for :py:func:`np.take_along_axis <numpy.take_along_axis>`.
+
+    See its docstring for more information.
+    """
+    if x.device != indices.device:
+        raise ValueError(f"Arrays from two different devices ({x.device} and {indices.device}) can not be combined.")
+    return Array._new(np.take_along_axis(x._array, indices._array, axis), device=x.device)
